/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "ByteShiftArray.h"
#include <QtAlgorithms>
#include <QtGlobal>

//------------------------------------------------------------------------------
// Name: ByteShiftArray(std::size_t size)
// Desc: constructor
//------------------------------------------------------------------------------
ByteShiftArray::ByteShiftArray(std::size_t size) : m_Data(new quint8[size]()), m_Size(size) {
}

//------------------------------------------------------------------------------
// Name: ByteShiftArray(const ByteShiftArray& other)
// Desc: copy constructor
//------------------------------------------------------------------------------
ByteShiftArray::ByteShiftArray(const ByteShiftArray& other) : m_Data(new quint8[other.m_Size]), m_Size(other.m_Size) {
	qCopy(&m_Data[0], &m_Data[m_Size], &other.m_Data[0]);
}

//------------------------------------------------------------------------------
// Name: operator=(const ByteShiftArray &other
// Desc: assignment operator
//------------------------------------------------------------------------------
ByteShiftArray &ByteShiftArray::operator=(const ByteShiftArray &other) {
	if(&other != this) {
		m_Size = other.m_Size;
		m_Data.reset(new quint8[m_Size]);
		qCopy(&m_Data[0], &m_Data[m_Size], &other.m_Data[0]);
	}
	return *this;
}

//------------------------------------------------------------------------------
// Name: ~ByteShiftArray()
// Desc: destructor
//------------------------------------------------------------------------------
ByteShiftArray::~ByteShiftArray() {
}

//------------------------------------------------------------------------------
// Name: shl()
// Desc: shifts data left one byte and shifts in a 0
//------------------------------------------------------------------------------
ByteShiftArray &ByteShiftArray::shl() {

	for(std::size_t i = 0; i < (m_Size - 1); ++i) {
		m_Data[i] = m_Data[i + 1];
	}
	
	m_Data[m_Size - 1] = 0;

	return *this;
}

//------------------------------------------------------------------------------
// Name: shr()
// Desc: shifts data right one byte and shifts in a 0
//------------------------------------------------------------------------------
ByteShiftArray &ByteShiftArray::shr() {
	for(std::size_t i = m_Size - 1; i >= 1; --i) {
		m_Data[i] = m_Data[i - 1];
	}
	
	m_Data[0] = 0;
	return *this;
}

//------------------------------------------------------------------------------
// Name: size() const
// Desc: returns size of this byte array
//------------------------------------------------------------------------------
std::size_t ByteShiftArray::size() const {
	return m_Size;
}

//------------------------------------------------------------------------------
// Name: operator[](std::size_t i)
// Desc: returns and l-value version of an element in the byte array
//------------------------------------------------------------------------------
quint8 &ByteShiftArray::operator[](std::size_t i) {
	return m_Data[i];
}

//------------------------------------------------------------------------------
// Name: operator[](std::size_t i)
// Desc: returns and r-value version of an element in the byte array
//------------------------------------------------------------------------------
quint8 ByteShiftArray::operator[](std::size_t i) const {
	return m_Data[i];
}

//------------------------------------------------------------------------------
// Name: data() const
// Desc: returns a read only pointer to the data this byte array holds
//------------------------------------------------------------------------------
const quint8 *ByteShiftArray::data() const {
	return m_Data.get();
}

//------------------------------------------------------------------------------
// Name: clear()
// Desc: zeros out the byte array
//------------------------------------------------------------------------------
void ByteShiftArray::clear() {
	qFill(&m_Data[0], &m_Data[m_Size], 0);
}

//------------------------------------------------------------------------------
// Name: operator<<(quint8 x)
// Desc: 
//------------------------------------------------------------------------------
ByteShiftArray &ByteShiftArray::operator<<(quint8 x) {
	shl();
	m_Data[m_Size - 1] = x;
	return *this;
}
