/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef ANALYZER_20080630_H_
#define ANALYZER_20080630_H_

#include "DebuggerPluginInterface.h"
#include "AnalyzerInterface.h"
#include "Types.h"
#include <QSet>

class QMenu;

class Analyzer : public QObject, public AnalyzerInterface, public DebuggerPluginInterface {
	Q_OBJECT
	Q_INTERFACES(DebuggerPluginInterface)
	Q_CLASSINFO("author", "Evan Teran")
	Q_CLASSINFO("url", "http://www.codef00.com")

public:
	Analyzer();
	
public:
	virtual QMenu *menu(QWidget *parent = 0);
	
private:
	virtual void privateInit();
	
public:
	virtual void analyze(const MemRegion &region);
	virtual FunctionMap functions(const MemRegion &region) const;
	virtual AddressCatagory catagory(edb::address_t address) const;
	virtual void invalidateAnalysis(const MemRegion &region);
	virtual void invalidateAnalysis();

private:
	void bonusStackFramesHelper(Function &info) const;
	void setFunctionTypesHelper(Function &info) const;
	void findFunctionCalls(const MemRegion &region, FunctionMap &results);
	void bonusStackFrames(FunctionMap &results);
	void bonusSymbols(const MemRegion &region, FunctionMap &results);
	void findCallsFromFunctions(const MemRegion &region, FunctionMap &results, bool known);
	void setFunctionTypes(FunctionMap &results);
	void bonusELFEntryPoint(const MemRegion &region, FunctionMap &results) const;
	QByteArray md5Region(const MemRegion &region) const;
	int findFunctionEnds(FunctionMap &results, const MemRegion &region, bool known, QSet<edb::address_t> &walkedFunctions);
	edb::address_t findFunctionEnd(const MemRegion &region, edb::address_t start_address, edb::address_t end_address, int &update_count, FunctionMap &results, edb::address_t &last_instruction, bool known);
	bool findContainingFunction(edb::address_t address, Function &function) const;
	bool isInsideKnown(const MemRegion &region, edb::address_t address);
	bool isStackFrame(edb::address_t address) const;
	void bonusMain(const MemRegion &region, FunctionMap &results) const;
	bool isThunk(edb::address_t address) const;

Q_SIGNALS:
	void updateProgress(int);

public Q_SLOTS:
	void doMenu();
	
private:
	struct RegionInfo {
		FunctionMap				analysis;
		QByteArray				md5;
		QSet<edb::address_t>	known_calls;
	};
	
	QMenu						*m_Menu;
	QMap<MemRegion, RegionInfo>	m_AnalysisInfo;
};

#endif
